from datetime import datetime
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import rcParams
rcParams['font.family'] = 'serif'
rcParams['font.sans-serif'] = 'Times'
rcParams['font.size'] = 6
from matplotlib import cm
from matplotlib.ticker import LinearLocator,ScalarFormatter, FormatStrFormatter, MaxNLocator
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec

import shutil

from function_pareto import *

#FUNCTIONS#######################################

def ensure_dir(f):
    if not os.path.exists(f):
        os.makedirs(f)
        
def paretofront(pareto_set_v,pareto_set_m,pareto_set_b,pareto_set_h,v,m,bhlrhoEF):
                      
    pareto_set_v   = append(pareto_set_v,array([v]))
    pareto_set_m   = append(pareto_set_m,array([m]))
    pareto_set_b   = append(pareto_set_b,array([bhlrhoEF[0]]))
    pareto_set_h   = append(pareto_set_h,array([bhlrhoEF[1]]))
    
    dominuje     =  array([])
    
    for i in range(0,len(pareto_set_v)):

        dominuje = append(dominuje,array([True]))

        for j in range(0,i-1):
            
            if ((pareto_set_v[i]>pareto_set_v[j]) and (pareto_set_m[i]>pareto_set_m[j])):
            
                dominuje[i] = False

        for j in range(i+1,len(pareto_set_v)):
            
            if ((pareto_set_v[i]>pareto_set_v[j]) and (pareto_set_m[i]>pareto_set_m[j])):
            
                dominuje[i] = False

    pareto_set_v  = extract(dominuje, pareto_set_v)
    pareto_set_m  = extract(dominuje, pareto_set_m)
    pareto_set_b  = extract(dominuje, pareto_set_b)
    pareto_set_h  = extract(dominuje, pareto_set_h)
            
    return pareto_set_v,pareto_set_m,pareto_set_b,pareto_set_h        

#FUNCTIONS#######################################

min_b,min_h =   0.1,0.1
max_b,max_h =   0.2,0.2

b           =   arange(min_b, max_b, 0.005)
h           =   arange(min_h, max_h, 0.005)
l           =   1.0
rho         =   7860.0
E           =   210.0e9
F           =   1.0e3

b, h        =   meshgrid(b,h)

max_v,min_m =   function(min_b,min_h,l,rho,E,F)
min_v,max_m =   function(max_b,max_h,l,rho,E,F)

v, m        =   function(b,h,l,rho,E,F)         

ms_1        =   3
ms_2        =   4

plt.figure()
plt.subplots_adjust(hspace=0.3,wspace=0.3)

sbplt_pareto      = plt.subplot2grid((2,4),(0,1), colspan=2, rowspan=1)
sbplt_v           = plt.subplot2grid((2,4),(1,0), colspan=2, rowspan=1, projection='3d')
sbplt_m           = plt.subplot2grid((2,4),(1,2), colspan=2, rowspan=1, projection='3d')

sbplt_v.view_init(50, -215)
sbplt_m.view_init(50, -215)

sbplt_v.plot_surface(1000*b,1000*h,1000*v,rstride=1,cstride=1,linewidth=0,cmap=cm.coolwarm,alpha=0.5)
sbplt_m.plot_surface(1000*b,1000*h,m,rstride=1,cstride=1,linewidth=0,cmap=cm.coolwarm,alpha=0.5)

sbplt_pareto.set_title('Pareto')
sbplt_pareto.set_xlabel('$v$ [mm]')
sbplt_pareto.set_ylabel('$m$ [kg]')
sbplt_pareto.grid(True)

sbplt_v.set_xlabel('$b$ [mm]')
sbplt_v.set_ylabel('$h$ [mm]')
sbplt_v.set_zlabel('$v$  [mm]')

sbplt_m.set_xlabel('$b$ [mm]')
sbplt_m.set_ylabel('$h$ [mm]')
sbplt_m.set_zlabel('$m$  [kg]')

#Animation#############

name_opti   =   'evolution_PARETO'

anim_dir    =   os.path.join(os.getcwd(),'animations')
anim_dir    =   os.path.join(anim_dir,name_opti)

ensure_dir(anim_dir)

opti_dir    =   os.path.join(os.getcwd(),name_opti)

zacatek     = True

for dirname, dirnames, filenames in os.walk(opti_dir):

    for subdirname in dirnames:

        subdir_path             =  os.path.join(dirname,subdirname)
        filename_file_input     =  os.path.join(subdir_path,'input_pareto.txt')
        filename_file_output    =  os.path.join(subdir_path,'output_pareto.txt')
        filename_figure         =  os.path.join(subdir_path,'figure_pareto.png')

        if os.path.exists(filename_file_input) and os.path.exists(filename_file_output) and ('Design' in subdirname):
            
            destination = os.path.join(anim_dir,subdirname+'.png')
            
            bhlrhoEF      = function_variable_read_from_file(filename_file_input)
            v,m           = function_variable_read_from_file(filename_file_output)

            if zacatek:
            
                pareto_set_v  = array([v])
                pareto_set_m  = array([m])
                pareto_set_b  = array([bhlrhoEF[0]])
                pareto_set_h  = array([bhlrhoEF[1]])
                
                zacatek   = False
            
            else:
            
                pareto_set_v,pareto_set_m,pareto_set_b,pareto_set_h=paretofront(pareto_set_v,pareto_set_m,pareto_set_b,pareto_set_h,v,m,bhlrhoEF)

            sbplt_pareto.plot(1000*v, m, 'ro', ms=ms_2)
            try:
                pareto_pareto
            except NameError:    
                pareto_pareto  = sbplt_pareto.plot(1000*pareto_set_v, pareto_set_m, 'go', ms=ms_2)
            else:
                pareto_pareto.pop(0).remove()
                pareto_pareto  = sbplt_pareto.plot(1000*pareto_set_v, pareto_set_m, 'go', ms=ms_2)

            sbplt_v.plot(1000*bhlrhoEF[0], 1000*bhlrhoEF[1], 1000*v, 'ro', ms=ms_1)
            try:
                pareto_v
            except NameError:
                pareto_v  = sbplt_v.plot(1000*pareto_set_b, 1000*pareto_set_h, 1000*pareto_set_v, 'go', ms=ms_1)
            else:
                pareto_v.pop(0).remove()
                pareto_v  = sbplt_v.plot(1000*pareto_set_b, 1000*pareto_set_h, 1000*pareto_set_v, 'go', ms=ms_1)            
                            
            sbplt_m.plot(1000*bhlrhoEF[0], 1000*bhlrhoEF[1], m, 'ro', ms=ms_1)
            try:
                pareto_m
            except NameError:    
                pareto_m  = sbplt_m.plot(1000*pareto_set_b, 1000*pareto_set_h, pareto_set_m, 'go', ms=ms_1)
            else:
                pareto_m.pop(0).remove()
                pareto_m  = sbplt_m.plot(1000*pareto_set_b, 1000*pareto_set_h, pareto_set_m, 'go', ms=ms_1)            
            
            sbplt_pareto.axis([1000*min_v,1000*max_v,min_m,max_m])
            
            sbplt_v.set_xlim3d(1000*min_b, 1000*max_b)
            sbplt_v.set_ylim3d(1000*min_h, 1000*max_h)
            sbplt_v.set_zlim3d(1000*min_v, 1000*max_v)
            
            sbplt_m.set_xlim3d(1000*min_b, 1000*max_b)
            sbplt_m.set_ylim3d(1000*min_h, 1000*max_h)
            sbplt_m.set_zlim3d(min_m, max_m)            
            
            plt.savefig(filename_figure, dpi=300, format='png')
            
            print 'Directory ',subdir_path,' -> ',destination
            shutil.copy(filename_figure, destination)