import matplotlib.pyplot as plt
import numpy as np

from mpl_toolkits.mplot3d import Axes3D

def f(x, A=1., B=1.):
    """
    Default function to be fitted to data:
    f = A * exp(-B*x)
    """
    return A * np.exp(-B * x)

def plot_box(A, B, Ain, Bin, xi, fun=f, N=11):
    """
    Plots contours of least-squares-sum in an interval of parameters A, B.

    A, B : actual values
    Ain, Bin : range to be plotted
    xi : values of x to sum over
    N : A/B-plotting interval division
    """
    if len(Ain) == 2:
        Aint = np.linspace(Ain[0], Ain[1], N)
    else:
        Aint = Ain

    if len(Bin) == 2:
        Bint = np.linspace(Bin[0], Bin[1], N)
    else:
        Bint = Bin

    fs = np.zeros((len(Aint), len(Bint)))
    for ii, Aii in enumerate(Aint):
        for jj, Bjj in enumerate(Bint):
            fs[ii, jj] = sum(
                [(f(x, A=A, B=B) - f(x, A=Aii, B=Bjj)) ** 2
                 for x in xi])

    fig = plt.figure()
    AA = np.array([Aint for Bj in Bint])
    BB = np.array([Bint for Ai in Aint]).T
    #ax = fig.add_subplot(111, projection='3d')
    print 'min(fs) =', np.min(fs), ' max(fs) =', np.max(fs)
    plt.contour(AA, BB, fs.T,
                np.logspace(max(np.log(np.min(fs)) / np.log(10), -14),
                            min(np.log(np.max(fs)) / np.log(10), 2),
                            41))
    #ax.plot_surface(AA, BB, fs, cmap=plt.cm.jet)
    #plt.colorbar()
    plt.xlabel('$A$')
    plt.ylabel('$B$')
    plt.grid()
    plt.plot(A, B, 'or')

    return fs

if __name__ == '__main__':
    plot_box(.6, 1.4,
             np.linspace(0, 2, 201),
             np.linspace(0, 4, 401),
             np.linspace(1, 3, 2),
    )

    plot_box(.6, 1.4,
             np.linspace(0, 2, 201),
             np.linspace(0, 4, 401),
             np.linspace(.5, 3, 5),
    )

    plt.show()
