import matplotlib.pyplot as plt
import numpy as np

from mpl_toolkits.mplot3d import Axes3D

def sumsq(fun, x, y):
    yf = np.array([fun(xi) for xi in x])
    return sum((y - yf) ** 2)

def sumsq_norm1(fun, x, y):
    def nfun(x):
        return 0

    return sumsq(fun, x, y) / sumsq(nfun, x, y)

def sumsq_norm2(fun, x, y):
    yf = np.array([fun(xi) for xi in x])
    return sum((y - yf) ** 2 / y ** 2)

def sumsq_norm3(fun, x, y):
    yf = np.array([fun(xi) for xi in x])
    return sum(1 - np.exp(-(y - yf) ** 2 / y ** 2))

if __name__ == '__main__':
    a = 1.2
    b = .8
    def tfun(x, A=a, B=b):
        return A * x ** B

    x = np.linspace(.2, 2, 10)
    y = np.array([tfun(xi) for xi in x])

    Na = 20
    Nb = 10
    La = 2.
    Lb = 1.
    ai = np.linspace(La / Na, La, Na)
    bj = np.linspace(Lb / Nb, Lb, Nb)

    f1 = np.zeros((Na, Nb))
    f2 = np.zeros((Na, Nb))
    for ii, aii in enumerate(ai):
        for jj, bjj in enumerate(bj):
            tiijj = lambda x: tfun(x, A=aii, B=bjj)
            f1[ii, jj] = sumsq_norm3(tiijj, x, y)
            f2[ii, jj] = sumsq_norm2(tiijj, x, y)

    AA = np.array([ai for b in bj]).T
    BB = np.array([bj for a in ai])
    scale = np.linspace(0, 1, 11)

    fig1 = plt.figure()
    ax1 = fig1.add_subplot(111, projection='3d')
    ax1.plot_wireframe(AA, BB, f1, cmap=plt.cm.jet)
#    plt.colorbar()
#    plt.contour(ai, bj, f1.T, scale)
#    plt.plot(a, b, '.')
    plt.title('$f = \sum(exp(\ldots / \ldots))$')
    plt.xlabel('$a$')
    plt.ylabel('$b$')
    ax1.set_zlim(0, 10)

    fig2 = plt.figure()
    ax2 = fig2.add_subplot(111, projection='3d')
    ax2.plot_wireframe(AA, BB, f2, cmap=plt.cm.jet)
#    plt.colorbar()
#    plt.contour(ai, bj, f2.T, scale)
#    plt.plot(a, b, '.')
    plt.title('$f = \sum (\ldots )$')
    plt.xlabel('$a$')
    plt.ylabel('$b$')
    plt.show()
